searchData={"items":[{"type":"module","title":"CheckerCab","doc":"Documentation for CheckerCab.\n\nThis documentation assumes these functions are used in the context of unit\ntests.","ref":"CheckerCab.html"},{"type":"function","title":"CheckerCab.assert_values_for/1","doc":"Compares the values of two maps for specified keys.\n\nWhen values do not match, this function will flunk the current test with\nexplicit information about the first value that did not match.\n\nThis function also accepts a list of fields to _not_ compare, and can be mixed\nwith the fields to compare.\n\nThis function assumes keys are atoms unless specified as `:string_keys`, and\nwill convert keys into a common type before comparing values (can compare\natom-keyed and string-keyed maps with same-named keys). Additionally,\n`:atom_keys` can be provided to be more explicit.","ref":"CheckerCab.html#assert_values_for/1"},{"type":"function","title":"Options - CheckerCab.assert_values_for/1","doc":"* `:convert_dates`\n  When `true`, will convert date-representing values to ISO-8601 formatted\nstrings.","ref":"CheckerCab.html#assert_values_for/1-options"},{"type":"function","title":"Examples - CheckerCab.assert_values_for/1","doc":"```","ref":"CheckerCab.html#assert_values_for/1-examples"},{"type":"function","title":"your_unit_test.exs - CheckerCab.assert_values_for/1","doc":"expected = %{key1: :value, key2: :value, key3: :value}\nactual = %{\"key1\" => :value, \"key2\" => :value, \"key3\" => :value}","ref":"CheckerCab.html#assert_values_for/1-your_unit_test-exs"},{"type":"function","title":"returns :ok when expected and actual match - CheckerCab.assert_values_for/1","doc":"assert_values_for(\n  expected: expected,\n  actual: {actual, :string_keys},\n  fields: [:key1, :key2, :key3]\n)\n```\n\nWith dates:\n\n```\nexpected = %{date: ~U[2021-12-17 03:08:36.579609Z], key2: :value, key3: :value}\nactual = %{\"date\" => \"2021-12-17T03:08:36.579609Z\", \"key2\" => :value, \"key3\" => :value}","ref":"CheckerCab.html#assert_values_for/1-returns-ok-when-expected-and-actual-match"},{"type":"function","title":"Will not flunk for different types of dates if they both convert to the - CheckerCab.assert_values_for/1","doc":"same ISO 8601 string\n\nassert_values_for(\n  expected: expected,\n  actual: {actual, :string_keys},\n  fields: [:date, :key2, :key3],\n  opts: [convert_dates: true]\n)\n```\n\nUsing `:skip_fields`:\n\n```\nexpected = %{key1: :value, key2: :value, key3: :value, value_that_wont_match: \"Panama\"}\nactual = %{key1: :value, key2: :value, key3: :value, value_that_wont_match: \"Manimal\"}","ref":"CheckerCab.html#assert_values_for/1-will-not-flunk-for-different-types-of-dates-if-they-both-convert-to-the"},{"type":"function","title":"Returns :ok when expected and actual match. Can exclude fields anticipated - CheckerCab.assert_values_for/1","doc":"to be different.\nassert_values_for(\n  expected: expected,\n  actual: actual,\n  fields: [:key1, :key2, :key3],\n  skip_fields: [:value_that_wont_match]\n)\n```","ref":"CheckerCab.html#assert_values_for/1-returns-ok-when-expected-and-actual-match-can-exclude-fields-anticipated"},{"type":"function","title":"CheckerCab.fields_for/1","doc":"Returns the keys of a map or struct.","ref":"CheckerCab.html#fields_for/1"},{"type":"function","title":"Examples - CheckerCab.fields_for/1","doc":"iex> fields_for(%{\"string_key1\" => :value, \"string_key2\" => :value})\n    [\"string_key1\", \"string_key2\"]\n\n    iex> fields_for(%{atom_key1: :value, atom_key2: :value})\n    [:atom_key1, :atom_key2]\n\nReturns a list of defined keys from struct arguments (`:__struct__` is not\nreturned)\n\n    iex> fields_for(%StructModule{})\n    [:key1, :key2, :key3]\n\nReturns a list of defined keys from `Ecto.Schema` arguments, but does not\nreturn virtual fields.\n\n    iex> fields_for(%EctoSchemaModule{})\n    [:id, :field1, :field2, :field3]\n\nAdditionally, the function accepts the module name of an `Ecto.Schema`.\nThis does not work with structs.\n\n    iex> fields_for(EctoSchemaModule)\n    [:id, :field1, :field2, :field3]","ref":"CheckerCab.html#fields_for/1-examples"},{"type":"type","title":"CheckerCab.comparable_input/0","doc":"","ref":"CheckerCab.html#t:comparable_input/0"},{"type":"type","title":"CheckerCab.inputs/0","doc":"","ref":"CheckerCab.html#t:inputs/0"},{"type":"type","title":"CheckerCab.option/0","doc":"","ref":"CheckerCab.html#t:option/0"},{"type":"extras","title":"CheckerCab","doc":"# CheckerCab\n\n`assert_values_for` and friends.\n\n[View the full documentation on\nHex](https://hexdocs.pm/checker_cab/api-reference.html).","ref":"readme.html"},{"type":"extras","title":"Explanation - CheckerCab","doc":"#","ref":"readme.html#explanation"},{"type":"extras","title":"What is Checker Cab? - CheckerCab","doc":"Checker Cab facilitates deep map comparisons within unit tests.\n\n#","ref":"readme.html#what-is-checker-cab"},{"type":"extras","title":"How Will Checker Cab Improve My Unit Tests? - CheckerCab","doc":"Checker Cab helps alleviate tedium in these testing scenarios:\n  * Selective comparison on fields between maps.\n  * Phoenix Controller tests where expected input is an atom-keyed map such as a\n  struct and expected output is a string-keyed map such as a JSON response, but\n  keys are otherwise the same name.\n  * Comparisons between DateTime values and ISO-8601-formatted strings as values\n    in otherwise-equivalent maps.\n  * Identify exactly which value did not match between maps with many keys.\n  * Update tests to account for new values added to a struct (or map used as a\n    record).\n\nWhen relying on this library for unit tests, it becomes easier to write more\nthorough tests with the same amount of effort or less.\n\n#","ref":"readme.html#how-will-checker-cab-improve-my-unit-tests"},{"type":"extras","title":"Example - CheckerCab","doc":"Assuming a `User` struct and a `Factory` module that may build parameters, a\ncontroller test may look like this:\n\n```elixir\n  test \"success: it returns a 200 and a newly updated `User`\", %{conn: conn, user: %User{id: id}} do\n    %User{} = expected_updates = MyApp.Factory.build(:user, id: id)\n\n    conn = post(conn, Routes.user_path(conn, :update), user: expected_user)\n    assert %{\"id\" => ^id} = json_response(conn, 200)[\"data\"]\n  end\n```\nThis is a nice basis for a test to exercise HTTP response codes, but this does\nnot assert that actual values have been set. The assertions could be added\nindividually, with an assertion made for each field in the `User` struct.\nHowever, this can bloat a test over time at the expense of test clarity. Let's\nsee it with Checker Cab instead.\n\n```elixir\n  test \"success: it returns a 200 and a newly updated `User`\", %{conn: conn, user: %User{id: id}} do\n    %User{} = expected_updates = MyApp.Factory.build(:user, id: id)\n\n    conn = post(conn, Routes.user_path(conn, :update), user: expected_user)","ref":"readme.html#example"},{"type":"extras","title":"new stuff - CheckerCab","doc":"assert returned_user = %{\"id\" => ^id} = json_response(conn, 200)[\"data\"]","ref":"readme.html#new-stuff"},{"type":"extras","title":"note: assert_values_for/1 and fields_for/1 are provided by CheckerCab. - CheckerCab","doc":"assert_values_for(\n      expected: expected_updates,\n      actual: {returned_user, :string_keys},\n      fields: fields_for(User)\n    )\n  end\n```\nRegardless of how many fields the `User` struct may have or have added to it,\nthe assertions lock down that the returned user will have all fields in the\n`User` struct and the values will be the same. The test is self-updating and\nwill assist in catching regressions if the `update` function begins to set other\nvalues or if the view code does not capture newly-added fields to the `User`\nschema.\n\nFor the sake of example, if the controller action under test returned a `User`\nwith the same keys and values differing only with the value for id, then the\ntest would fail and the output may look like the example below:\n\n```\n  1) test success: it returns a 200 and a newly updated User (UserControllerTest)\n     test/user_controller_test.exs:294\n     There were issues with the comparison:\n\n     Values did not match for:\n       field: :id\n         expected: \"1ee3e9c4-fa81-4612-a13a-c554e5a3d438\"\n         actual: \"a2ab48fa-96fb-41e5-8f2d-2f94f47fef91\"\n     code: assert_values_for(expected: expected_updates, actual: {returned_user :string_keys}, fields: fields_for(User))\n     stacktrace:","ref":"readme.html#note-assert_values_for-1-and-fields_for-1-are-provided-by-checkercab"},{"type":"extras","title":"stacktrace here - CheckerCab","doc":"```\nAs a different example, lets assume the same unit test. While adding a new\nfeature, we add a `new_key` field to our `User` struct, but forget to add\n`new_key` to our `Factory` module. Checker Cab should detect the mismatch and\nalert the developer with following output:\n\n```\n  1) test success: it returns a 200 and a newly updated User (UserControllerTest)\n     test/user_controller_test.exs:294\n     There were issues with the comparison:\n\n     Key(s) missing:\n       field: :new_key didn't exist in expected\n     code: assert_values_for(expected: expected_updates, actual: {returned_user :string_keys}, fields: fields_for(User))\n     stacktrace:","ref":"readme.html#stacktrace-here"},{"type":"extras","title":"stacktrace here - CheckerCab","doc":"```","ref":"readme.html#stacktrace-here"},{"type":"extras","title":"Installation - CheckerCab","doc":"Add it to your deps.\n\n```elixir\ndef deps do\n  [","ref":"readme.html#installation"},{"type":"extras","title":"check hex.pm for the latest version - CheckerCab","doc":"{:checker_cab, \"~> 1.0.1\", runtime: false, only: [:test]},\n  ]\nend\n```","ref":"readme.html#check-hex-pm-for-the-latest-version"},{"type":"extras","title":"Integrating into a test suite - CheckerCab","doc":"Import `CheckerCab` to your test case file:\n```elixir","ref":"readme.html#integrating-into-a-test-suite"},{"type":"extras","title":"test/support/test_case.ex - CheckerCab","doc":"defmodule YourApp.TestCase do\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      import CheckerCab\n    end\n  end\nend\n```\n\nEnsure the test case file is compiled for the `test` environment:\n\n```elixir","ref":"readme.html#test-support-test_case-ex"},{"type":"extras","title":"mix.exs - CheckerCab","doc":"defmodule YourApp.MixProject\n  use Mix.Project\n\n  def project do\n    [\n      app: :your_app,\n      version: \"0.1.0\",\n      elixir: \"~> 1.13\",\n      start_permanent: Mix.env() == :prod,\n      deps: deps(),\n      elixirc_paths: elixirc_paths(Mix.env()),\n    ]\n  end","ref":"readme.html#mix-exs"},{"type":"extras","title":"skipping for brevity - CheckerCab","doc":"defp elixirc_paths(:test), do: [\"lib\", \"test/support\"]\n  defp elixirc_paths(_), do: [\"lib\"]\nend\n```\n\nAnd finally, ensure the test case file is used in a test:\n\n```elixir","ref":"readme.html#skipping-for-brevity"},{"type":"extras","title":"test/your_app/contrived_example_test - CheckerCab","doc":"defmodule YourApp.ContrivedExampleTest do\n  use YourApp.TestCase","ref":"readme.html#test-your_app-contrived_example_test"},{"type":"extras","title":"tests go here. - CheckerCab","doc":"end\n\n```\n\nThat's it. You're ready to take advantage of the splendors of `CheckerCab`. Honk Honk 🚕","ref":"readme.html#tests-go-here"},{"type":"extras","title":"Contributing - CheckerCab","doc":"#","ref":"readme.html#contributing"},{"type":"extras","title":"Releasing a new version - CheckerCab","doc":"To release a new version of this library, you have to\n\n  * Bump the version\n  * Update the changelog\n  * Release on Hex\n\n##","ref":"readme.html#releasing-a-new-version"},{"type":"extras","title":"Updating version and changelog - CheckerCab","doc":"To bump the version, update it in [`mix.exs`](./mix.exs). We use semantic versioning (`MAJOR.MINOR.PATCH`) which means:\n\n  * Bump the `MAJOR` version only if there are breaking changes (first get approval from the maintainers)\n  * Bump the `MINOR` version if you introduced new features\n  * Bump the `PATCH` version if you fixed bugs\n\nIn the same code change that updates the version (such as a PR), also update the [`CHANGELOG.md`](./CHANGELOG.md) file with a new entry.","ref":"readme.html#updating-version-and-changelog"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\n## ## [1.2.1] - 2024-02-11\n- Update documentation config for better hex_doc experience\n\n## [1.2.0] - 2024-02-07\n- Update all dependencies\n- Support Elixir 1.15.x and 1.16.x\n- Support OTP 26\n- Update testing approach to work with newer Elixir and OTP\n\n## [1.1.0] - 2022-12-12\n- Support Elixir 1.14.x and OTP 25\n- Increase test output to show all issues from call to `assert_values_for/1`.\n\n## [1.0.1] - 2022-11-17\n- Add improved failure output: catch and batch failures\n\n## [1.0.0] - 2022-10-20\n- Update README\n- Make public\n## [0.2.1] - 2021-12-16\n- Update documentation\n- Rename test file from `cookie_cutter_test.exs` to `checker_cab_test.exs`.\n\n## [0.2.0] - 2021-12-03\n- Update fields_for to strip virtual fields for Ecto schemas.\n\n## [0.1.1] - 2021-12-02\n- Add a LICENSE.\n- Add package metadata.\n\n## [0.1.0] - 2021-12-02\n- Update README\n- Add a CHANGELOG.md\n- Add `assert_values_for/1` and helper functions.","ref":"changelog.html"},{"type":"extras","title":"LICENSE","doc":"Copyright (c) 2021 Shimmur, Inc.\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in\nall copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN\nTHE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}